searchData={"content_type":"text/markdown","items":[{"doc":"","ref":"TestCircuit.MyCircuit.html","title":"TestCircuit.MyCircuit","type":"module"},{"doc":"","ref":"TestCircuit.MyCircuit.html#error_threshold/0","title":"TestCircuit.MyCircuit.error_threshold/0","type":"function"},{"doc":"","ref":"TestCircuit.MyCircuit.html#execute/1","title":"TestCircuit.MyCircuit.execute/1","type":"function"},{"doc":"","ref":"TestCircuit.MyCircuit.html#reset_timeout/0","title":"TestCircuit.MyCircuit.reset_timeout/0","type":"function"},{"doc":"","ref":"TestCircuit.MyCircuit.html#run/1","title":"TestCircuit.MyCircuit.run/1","type":"function"},{"doc":"# Virgil\n\nVirgil is an Elixir library designed to abstract and simplify the utilization of internal circuit breakers within applications. Its primary function is to act as a guardian, ensuring uninterrupted operation even in the face of recurrent errors.","ref":"Virgil.html","title":"Virgil","type":"module"},{"doc":"In modern applications, errors during requests are inevitable. Recurrent errors, however, can lead to service degradation, impacting user experience. Virgil addresses this challenge by seamlessly integrating with circuit breakers, providing a robust solution to handle such situations.","ref":"Virgil.html#module-overview","title":"Overview - Virgil","type":"module"},{"doc":"Circuit breakers are a mechanism designed to control requests to a service. Their primary purpose is to prevent overload and cascading failures. When a defined error threshold is exceeded, the circuit breaker \"opens,\" preventing further requests for a specified time period.","ref":"Virgil.html#module-circuit-breakers-a-brief-overview","title":"Circuit Breakers: A Brief Overview - Virgil","type":"module"},{"doc":"Virgil monitors requests and error rates. When recurrent errors are detected, it takes action by opening the circuit, temporarily halting requests. Once the defined time period has elapsed, normal operation is resumed.","ref":"Virgil.html#module-how-virgil-works","title":"How Virgil Works - Virgil","type":"module"},{"doc":"- **Reliability:** Ensures consistent performance, even in the presence of recurrent errors.\n- **Simplicity:** Abstracts complex circuit breaker logic for ease of use.\n- **Efficiency:** Prevents unnecessary requests during error-prone periods.","ref":"Virgil.html#module-benefits","title":"Benefits - Virgil","type":"module"},{"doc":"- **Microservices Architectures:** Ensure service resilience in a distributed environment.\n- **High Traffic Scenarios:** Safeguard against overload and cascading failures.\n- **Integration with Unreliable APIs:** Maintain reliability in the face of unpredictable external services.","ref":"Virgil.html#module-use-cases","title":"Use Cases - Virgil","type":"module"},{"doc":"Circuit implementation","ref":"Virgil.Circuit.html","title":"Virgil.Circuit","type":"behaviour"},{"doc":"","ref":"Virgil.Circuit.html#c:run/1","title":"Virgil.Circuit.run/1","type":"callback"},{"doc":"","ref":"Virgil.Circuit.html#t:t/0","title":"Virgil.Circuit.t/0","type":"type"},{"doc":"Virgil Manager callbacks","ref":"Virgil.Manager.html","title":"Virgil.Manager","type":"behaviour"},{"doc":"Closes the circuit, it means the circuit will execute every time it is called","ref":"Virgil.Manager.html#c:close/1","title":"Virgil.Manager.close/1","type":"callback"},{"doc":"Increments the circuit error counter","ref":"Virgil.Manager.html#c:increment_error_counter/1","title":"Virgil.Manager.increment_error_counter/1","type":"callback"},{"doc":"Validates if the circuit is closed","ref":"Virgil.Manager.html#c:is_closed?/1","title":"Virgil.Manager.is_closed?/1","type":"callback"},{"doc":"Opens the circuit, it will not execute until the counter was completely cleaned up and the manager opens it again","ref":"Virgil.Manager.html#c:open/1","title":"Virgil.Manager.open/1","type":"callback"},{"doc":"","ref":"Virgil.Manager.html#t:circuit_name/0","title":"Virgil.Manager.circuit_name/0","type":"type"},{"doc":"# Virgil\n\nVirgil is an Elixir library designed to abstract and simplify the utilization of internal circuit breakers within applications. Its primary function is to act as a guardian, ensuring uninterrupted operation even in the face of recurrent errors.","ref":"readme.html","title":"Virgil","type":"extras"},{"doc":"If [available in Hex](https://hex.pm/docs/publish), the package can be installed\nby adding `virgil` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:virgil, \"~> 1.0.2\"}\n  ]\nend\n```\n\nDocumentation can be generated with [ExDoc](https://github.com/elixir-lang/ex_doc)\nand published on [HexDocs](https://hexdocs.pm). Once published, the docs can\nbe found at  .","ref":"readme.html#installation","title":"Installation - Virgil","type":"extras"},{"doc":"#","ref":"readme.html#custom-circuit-example","title":"Custom circuit example - Virgil","type":"extras"},{"doc":"You can define a module that represents an application circuit following the example:\n\n```elixir\ndefmodule MyCustomCircuit do\n  @moduledoc false\n\n  # error_threshold: max number of tries until the circuit be openned\n  # reset_timeout: timeout in seconds until the manager close circuit back\n  use Virgil.Circuit,\n    error_threshold: 5,\n    reset_timeout: 10\n\n  def run(%{document_number: document}) do\n    response = Tesla.get(\"https://my-bureau-api.com/\", query: [document: document])\n\n    case response.status do\n      200 -> {:ok, response}\n      401 -> {:error, :unauthorized}\n      500 -> {:error, :api_error}\n    end\n  end\nend\n```\n\n#","ref":"readme.html#defining-circuit-module","title":"Defining circuit module - Virgil","type":"extras"},{"doc":"You can execute the circuit pipeline using the `MyCustomCircuit.execute/1` function.\nThis function receives one argument and pass it to the defined `MyCustomCircuit:run/1` function.\n\n```elixir\n%{document_number: \"11122233300\", name: \"John Doe\"}\n|> MyCustomCircuit.execute()\n```\n\n#","ref":"readme.html#executing-circuit","title":"Executing circuit - Virgil","type":"extras"},{"doc":"```mermaid\nflowchart TD\n    A[Circuit] -->|execute| C{successful response?}\n    C -->|Yes| D[Successful telemetry event]\n    C -->|No| E[Failure telemetry event]\n    E --> F{Number of failures reached threshold?}\n    F -->|Yes| G[Open circuit]\n    F -->|No| H[Increase Number of failures]\n    G --> I[Schedule process to close circuit back]\n```","ref":"readme.html#circuit-flow","title":"Circuit Flow - Virgil","type":"extras"}]}